@@ -3,7 +3,7 @@ package Contize;
 use strict;
 use Carp;
 
-our $VERSION = '0.2';
+our $VERSION = '0.3';
 
 our $AUTOLOAD;
 
@@ -13,98 +13,39 @@ Contize - Help an object be a continuation thingie (suspendable)
 
 =head1 SYNOPSIS
 
-  #!/usr/bin/perl
+  # Contize an object
+  my $obj = new Contize(new Object);
 
-  package WebGuess;
-  use strict;
-  use Contize;
-
-  sub new {
-    my $self = {};
-    bless $self;
-    $self = new Contize($self);
-    return $self;
+  # An output-and-suspend method
+  sub output {
+    my ($self, $msg) = @_;
+    print '<html><body><form method=post>'
+    print $msg;
+    print '</form></body></html>';
+    $self->suspend;
   }
 
-  sub setNumber {
+  # An input method
+  sub input {
+    my ($self, $msg) = @_;
+    $self->output(
+        $msg
+      . '<input type="text" name="response">'
+    );
+    $q = new CGI;
+    return $q->param('response');
+  }
+
+  # Now the magical bit!
+  sub addTwo {
     my $self = shift;
-    $self->{number} = int(rand(100)) + 1;
+    my $a = $self->input("Enter first number:");
+    my $b = $self->input("Enter second number:");
+    $self->output("Total of \$a + \$b = " . ($a + $b));
   }
 
-  sub display {
-    my ($self, $content) = @_;
-    print $content;
-    $self->suspend();
-  }
-
-  sub getNum {
-    my $self = shift;
-    $self->display(<<"END");
-      <form method=POST">
-        Enter Guess: <input name="num">
-        <input type=submit value="Guess">
-        <input type=submit name="done" value="Done">
-        <br>
-      </form>
-  END
-    return $::q->param('num');
-  }
-
-  # ***********************************
-  # **** THIS IS THE IMPORTANT BIT ****
-  # ***********************************
-  sub run {
-    my $self = shift;
-    $self->setNumber();
-    my $guess;
-    my $tries = 0;
-    print
-      "Hi! I'm thinking of a number from 1 to 100...
-      can you guess it?<br>\n";
-    do {
-      $tries++;
-      $guess = $self->getNum();
-      print "It is smaller than $guess.<br>\n" if($guess > $self->{number});
-      print "It is bigger than $guess.<br>\n" if($guess < $self->{number});
-    } until ($guess == $self->{number});
-    print "You got it! My number was in fact $self->{number}.<br>\n";
-    print "It took you $tries tries.<br>\n";
-  }
-
-  package Main;
-
-  use strict;
-  use CGI;
-  use CGI::Session;
-  use Data::Dumper;
-
-  # Set up the CGI session and print the header
-  $::q = new CGI();
-  my $session = new CGI::Session(undef, $::q, {Directory=>'/tmp'});
-  print $session->header();
-
-  # If there is a guess object in the session use it, otherwise create a new
-  # WebGuess object and Contize it.
-  my $g = $session->param('guess') || new WebGuess();
-
-  # Fix stuff -- most importantly the Data::Dumper version of the object doesn't
-  # get recreated correctly (I don't know why)... so to work around it I re-eval
-  # the thing. And we must reset the callstack and the callcount.
-  my $VAR1;
-  eval(Dumper($g));
-  $g = $VAR1;
-  $g->resume();
-
-  # Add the WebGuess object to the session
-  $session->param('guess', $g);
-
-  # Enter the main loop of the WebGuess object
-  until($::q->param('done')) {
-    $g->run();
-  }
-
-  # We won't get here until that exits cleanly (never!)
-  print "Done.";
+  # This example would be completed with a wrapper script to save/restore the
+  # object to disk between runs. See the WebGuess example
 
 =head1 DESCRIPTION
 
@@ -214,13 +155,13 @@ sub AUTOLOAD {
   # Figure out the method's full name
   my $method = (ref $self->{_child}) . "::$name";
   if($self->{_child}->can($method)) {
+
     # Keep track of this invocation through our internal stacks
     push @{$self->{_callstack}}, $name;
     my $callstack = "@{$self->{_callstack}}";
     my $count = ++$self->{_callstack_count}{$callstack};
     push @{$self->{_callstack}}, $count;
 
-
     # Check to see if we should cache the result
     if(grep {$_ eq $name} @{$self->{_nocache}}) {
       # We should NOT cache the result.
@@ -228,13 +169,15 @@ sub AUTOLOAD {
     } else {
       $callstack = "@{$self->{_callstack}}";
       if(exists $self->{_cache}{$callstack}) {
-        # We've already chached this call, lets just return it
+        # We've already cached this call, lets just return it
         $val = $self->{_cache}{$callstack};
       } else {
         # We've never done this before, lets run it...
         $val = $self->$method(@args);
         # Cache all method calls (direct AND inherited)
         $self->{_cache}{$callstack} = $val;
+
+        use Data::Dumper;
       }
     }
     pop @{$self->{_callstack}}; # The num
@@ -260,11 +203,13 @@ sub suspend {
   my $retval = shift;
   my $callstack = "@{$self->{_callstack}}";
   $self->{_cache}{$callstack} = $retval;
+  #$self->{_child}->cleanup() if ($self->{_child}->can('cleanup'));
+  $self->cleanup();
   exit;
 }
 
 
-=item $thingie->reset()
+=item $thingie->resume()
 
 Reset the thingie so that it will be re-run. This clears the callstack and the
 callstack_count so that it will begin returning cached results.
@@ -309,7 +254,7 @@ L<Coro::Cont>, L<http://thelackthereof.org/wiki.pl/Contize>
 =head1 AUTHOR
 
   Brock Wilcox <awwaiid@thelackthereof.org>
-  L<http://thelackthereof.org/>
+  http://thelackthereof.org/
 
 =head1 COPYRIGHT
 
